# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-07 14:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_data_collections', '0004_webformpluginmodel'),
    ]

    operations = [
        migrations.AlterField(
            model_name='webformpluginmodel',
            name='title',
            field=models.CharField(blank=True, default='', help_text='Optional. If not set the web form title will be used.', max_length=255, verbose_name='Title'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-07 12:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('pcart_data_collections', '0003_datacollection_reversed_ordering'),
    ]

    operations = [
        migrations.CreateModel(
            name='WebFormPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='pcart_data_collections_webformpluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('show_title', models.BooleanField(default=True, verbose_name='Show title')),
                ('title', models.CharField(blank=True, default='', help_text='Optional. If not set the collection title will be used.', max_length=255, verbose_name='Title')),
                ('template_name', models.CharField(blank=True, default='default', max_length=100, verbose_name='Template name')),
                ('web_form', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='pcart_data_collections.WebForm', verbose_name='Web form')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



from django.utils.translation import ugettext_lazy as _
from django.forms import (
    IntegerField,
    FloatField,
    ChoiceField,
    MultipleChoiceField,
    CharField,
    EmailField,
    BooleanField,
    FileField,
    DateField,
    HiddenInput,
    ValidationError,
    ModelForm,
    Form,
)
from django.forms.widgets import (
    TextInput,
    Textarea,
    ClearableFileInput,
)
from django.conf import settings
from django.contrib.admin import widgets as admin_widgets
from django_ace import AceWidget

from datetimewidget import widgets as datetime_widgets
from djangocms_text_ckeditor.widgets import TextEditorWidget
from captcha.fields import ReCaptchaField
from .models import DataCollectionField, DataRecord, WebFormPluginModel


class DataRecordImportForm(Form):
    source = FileField(label=_('File'), required=True)


class DataCollectionFieldAdminForm(ModelForm):
    class Meta:
        model = DataCollectionField
        fields = '__all__'
        widgets = {
            'help_text': TextInput(attrs={'class': 'vTextField'}),
        }


class DataRecordForm(ModelForm):

    class Meta:
        model = DataRecord
        fields = ['title', 'active']

    WIDGETS = {
        DataCollectionField.TEXT: Textarea,
        DataCollectionField.CODE: AceWidget(
            wordwrap=False,
            width="100%",
            height="300px",
            showprintmargin=True),
        DataCollectionField.FILE: ClearableFileInput,
        DataCollectionField.DATE: admin_widgets.AdminDateWidget,
    }
    FIELD_CLASSES = {
        DataCollectionField.TEXT: CharField,
        DataCollectionField.EMAIL: EmailField,
        DataCollectionField.WYSIWYG: CharField,
        DataCollectionField.CODE: CharField,
        DataCollectionField.STRING: CharField,
        DataCollectionField.INTEGER: IntegerField,
        DataCollectionField.FLOAT: FloatField,
        DataCollectionField.CHOICE: ChoiceField,
        DataCollectionField.MULTIPLE_CHOICE: MultipleChoiceField,
        DataCollectionField.FILE: FileField,
        DataCollectionField.BOOLEAN: BooleanField,
        DataCollectionField.DATE: DateField,
    }

    error_messages = {
        'title_is_not_unique': _('Title must be unique.'),
        'title_same_multiple_found':
        _('Fix your database. More than one same Title found.'),
    }

    def _get_field_widget(self, field):
        if field.field_type == DataCollectionField.CODE:
            if field.choices:
                _mode = field.choices.splitlines()[0]
            else:
                _mode = 'default'
            return self.WIDGETS.get(field.field_type)(mode=_mode)
        else:
            return self.WIDGETS.get(field.field_type)

    def _get_field_object(self, field):
        _field_class = self.FIELD_CLASSES.get(field.field_type)
        _widget = self._get_field_widget(field)
        _field_obj = _field_class(
            label=field.title,
            required=field.required,
            help_text=field.help_text,
            widget=_widget
        )

        if field.field_type == DataCollectionField.CHOICE:
            _choices = [(t, t) for t in field.choices.splitlines()]
            _choices = [('', '-----')] + _choices
            _field_obj.choices = _choices
        elif field.field_type == DataCollectionField.MULTIPLE_CHOICE:
            _choices = [(t, t) for t in field.choices.splitlines()]
            _field_obj.choices = _choices
        elif field.field_type == DataCollectionField.FILE:
            _field_obj.is_file_upload = True
        elif field.field_type == DataCollectionField.BOOLEAN:
            _field_obj.is_checkbox = True
        return _field_obj

    def clean_name(self):
        name = self.cleaned_data['name']
        if self.data_collection.unique_names:
            try:
                record = DataRecord.objects.get(
                    data_collection=self.data_collection,
                    name__exact=name,
                )
                if hasattr(self, 'instance'):
                    if self.instance.pk == record.pk:
                        # Trying to save available record
                        return name
                raise ValidationError(
                    self.error_messages['name_is_not_unique'])
            except DataRecord.DoesNotExist:
                return name
            except DataRecord.MultipleObjectsReturned:
                raise ValidationError(
                    self.error_messages['name_same_multiple_found'])
        else:
            return name

    def __init__(self, *args, **kwargs):
        from django.core.files.storage import default_storage

        class FakeField:
            storage = default_storage

        data_collection = kwargs.pop('data_collection')
        self.data_collection = data_collection
        self.request = kwargs.pop("request")
        super().__init__(*args, **kwargs)

        collection_fields = self.get_collection_fields()

        self.fields['active'].is_checkbox = True
        if data_collection.title_field_label:
            self.fields['title'].label = data_collection.title_field_label
        if data_collection.active_field_label:
            self.fields['active'].label = data_collection.active_field_label

        for field in collection_fields:
            _field_name = field.name
            self.fields[_field_name] = self._get_field_object(field)

            # Initialize fields if an instance is specified
            if 'instance' in kwargs:
                data_record = kwargs.get('instance')
                custom_data = data_record.data
                _initial = custom_data.get(_field_name)
                if _initial is not None:
                    self.fields[_field_name].initial = _initial
                    if field.field_type == DataCollectionField.FILE:
                        from django.db.models.fields.files import FieldFile
                        _ff = FieldFile(None, FakeField, _initial)
                        self.fields[_field_name].initial = _ff

            # Initialize fields via GET arguments
            if _field_name in self.request.GET:
                self.fields[_field_name].initial = self.request.GET.get(_field_name, '')

    def get_collection_fields(self):
        collection_fields = DataCollectionField.objects.filter(
            data_collection=self.data_collection,
        ).order_by('collection_order')
        return collection_fields

    def get_upload_path(self, data_record, field):
        import os
        from django.utils.text import slugify
        _base_upload_path = os.path.join(
            'data-collections',
            self.data_collection.name,
            slugify(data_record.name),
        )
        _upload_path = os.path.join(
            _base_upload_path,
            field.name,
        )
        return _upload_path

    def save(self, force_insert=False, force_update=False, commit=True):
        import os
        from django.core.files.storage import default_storage
        from django.core.files.base import File
        from django.utils.text import slugify

        data_record = super().save(commit=False)
        # Save record properly
        custom_data = {}

        if hasattr(self, 'instance'):
            record_id = self.instance.pk
        else:
            record_id = None

        collection_fields = self.get_collection_fields()

        for field in collection_fields:
            if field.field_type == DataCollectionField.FILE:
                if field.name in self.request.FILES:
                    _upload_path = self.get_upload_path(data_record, field)
                    _file = self.request.FILES[field.name]
                    _file_name = self.cleaned_data[field.name].name
                    _t = os.path.splitext(_file_name)
                    _file_name = '%s%s' % (slugify(_t[0]), _t[1])
                    _dest_name = os.path.join(
                        _upload_path,
                        _file_name)
                    _path = default_storage.save(
                        _dest_name, File(_file))
                    custom_data[field.name] = _path
                elif ('%s-clear' % field.name) in self.request.POST:
                    _path = data_record.data.get(field.name)
                    if _path:
                        default_storage.delete(_path)
                else:
                    if record_id:
                        if isinstance(data_record.data, dict):
                            _t = data_record.data.get(field.name)
                            # print(_t, str(self.cleaned_data[field.name]))
                            if _t not in ['None', None]:
                                custom_data[field.name] = _t
                            else:
                                # print(self.cleaned_data)
                                custom_data[field.name] = \
                                    str(self.cleaned_data[field.name])
                        else:
                            custom_data[field.name] = \
                                str(self.cleaned_data[field.name])
            elif field.field_type == DataCollectionField.DATE:
                custom_data[field.name] = str(self.cleaned_data[field.name])
                """
                Use something like that for query:
                DataRecord.objects.raw(
                    'select * from datacollections_datarecord where '
                    'to_date(data->>\'birthday\',\'YYYY-MM-DD\') '
                    'between \'1981-10-10\' and \'1981-12-12\'')
                """
            elif field.field_type == DataCollectionField.INTEGER:
                custom_data[field.name] = int(self.cleaned_data[field.name])
            elif field.field_type == DataCollectionField.FLOAT:
                custom_data[field.name] = float(self.cleaned_data[field.name])
            elif field.field_type == DataCollectionField.BOOLEAN:
                custom_data[field.name] = bool(self.cleaned_data[field.name])
            else:
                custom_data[field.name] = self.cleaned_data[field.name]
        data_record.data = custom_data
        data_record.data_collection = self.data_collection

        if commit:
            data_record.save()
        return data_record


class DataRecordSiteForm(DataRecordForm):

    WIDGETS = {
        DataCollectionField.TEXT: Textarea,
        DataCollectionField.WYSIWYG: TextEditorWidget,
        DataCollectionField.CODE: AceWidget(
            wordwrap=False,
            width="100%",
            height="300px",
            showprintmargin=True),
        DataCollectionField.FILE: ClearableFileInput,
        DataCollectionField.DATE: datetime_widgets.DateWidget(
            usel10n=True, bootstrap_version=3,
            options={'format': 'yyyy-mm-dd'}),
    }

    def __init__(self, *args, **kwargs):
        self.web_form = kwargs.pop('web_form')
        kwargs.update({
            'data_collection': self.web_form.data_collection
        })
        super().__init__(*args, **kwargs)

        # Remove standard fields if required
        field_list = self.web_form.fields.split(',')
        if 'active' not in field_list:
            del self.fields['active']
        if 'title' not in field_list:
            del self.fields['title']

        hidden_fields = self.web_form.hidden_fields.split(',')
        for f in hidden_fields:
            if f in field_list:
                self.fields[f].widget = HiddenInput()

        if self.web_form.show_captcha:
            self.fields['captcha'] = ReCaptchaField()

    def get_collection_fields(self):
        field_list = self.web_form.fields.split(',')

        collection_fields = super().get_collection_fields()
        return collection_fields.filter(name__in=field_list)


class WebFormPluginForm(ModelForm):
    model = WebFormPluginModel

    class Meta:
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        super(WebFormPluginForm, self).__init__(*args, **kwargs)
        self.fields['template_name'] = ChoiceField(
            choices=settings.PCART_WEB_FORM_TEMPLATES.items(),
            label=_('Template name'),
        )



from django.contrib.gis import admin
from django.utils.translation import ugettext_lazy as _
from adminsortable.admin import SortableStackedInline, NonSortableParentAdmin
from .models import (
    DataCollection,
    DataCollectionField,
    DataRecord,
    WebForm,
    MailHook,
)
from .forms import DataCollectionFieldAdminForm


class DataCollectionFieldInline(SortableStackedInline):
    model = DataCollectionField
    form = DataCollectionFieldAdminForm
    fieldsets = (
        (None, {
            'fields': ['name', 'title', 'field_type', 'required'],
        }),
        (_('Extra'), {
            'fields': ['choices', 'help_text', 'admin_list_display'],
            'classes': ('collapse',),
        }),
    )
    extra = 1


class DataCollectionAdmin(NonSortableParentAdmin):
    list_display = ['title', 'get_records_count', 'records_manage_link']
    inlines = [DataCollectionFieldInline]

    def get_records_count(self, obj):
        return obj.records.count()
    get_records_count.short_description = _('Records count')

    def records_manage_link(self, obj):
        from django.core.urlresolvers import reverse
        return '<a class="button" href="%s">%s</a>' % (
            reverse(
                'admin:datacollection_records_list',
                args=[obj.pk]),
            _('Data Records'),
        )
    records_manage_link.short_description = _('Actions')
    records_manage_link.allow_tags = True

    def records_list_view(self, request, collection_id):
        from django.shortcuts import render, get_object_or_404
        data_collection = get_object_or_404(
            DataCollection, id=collection_id)
        data_records = data_collection.get_records()

        context = dict(
           # Include common variables for rendering the admin template.
           self.admin_site.each_context(request),
           data_collection=data_collection,
           data_records=data_records,
        )
        return render(
            request,
            'admin/data_collections/datacollection/datarecords_list.html',
            context)

    def record_change_view(self, request, collection_id, record_id=None):
        from django.shortcuts import render, redirect, get_object_or_404
        from django.core.urlresolvers import reverse
        from .forms import DataRecordForm

        data_collection = get_object_or_404(
            DataCollection, id=collection_id)

        data_record = None
        if record_id:
            data_record = get_object_or_404(
                DataRecord,
                data_collection=data_collection,
                id=record_id)

        if request.method == 'POST':
            if record_id:
                form = DataRecordForm(
                    request.POST,
                    request.FILES,
                    instance=data_record,
                    data_collection=data_collection,
                    request=request,
                )
            else:
                form = DataRecordForm(
                    request.POST,
                    request.FILES,
                    data_collection=data_collection,
                    request=request,
                )
            if form.is_valid():
                data_record = form.save()

                if '_save' in request.POST:
                    return redirect(
                        reverse(
                            'admin:datacollection_records_list',
                            args=[data_collection.pk]))
                elif '_addanother' in request.POST:
                    return redirect(
                        reverse(
                            'admin:datacollection_records_add',
                            args=[data_collection.pk]))
                elif '_continue' in request.POST:
                    return redirect(
                        reverse(
                            'admin:datacollection_records_change',
                            args=[data_collection.pk, data_record.pk]))
        else:
            if record_id:
                form = DataRecordForm(
                    instance=data_record,
                    data_collection=data_collection,
                    request=request,
                )
            else:
                form = DataRecordForm(
                    data_collection=data_collection,
                    request=request,
                )

        context = dict(
           # Include common variables for rendering the admin template.
           self.admin_site.each_context(request),
           data_collection=data_collection,
           has_file_field=form.is_multipart(),
           show_save_and_add_another=True,
           show_save_and_continue=True,
           show_delete_link=False,
           form=form,
        )
        if record_id:
            context.update({
                'data_record': data_record,
                'show_delete_link': True,
            })
        return render(
            request,
            'admin/data_collections/datacollection/datarecord_change.html',
            context)

    def record_delete_view(self, request, collection_id, record_id):
        from django.shortcuts import redirect, get_object_or_404
        from django.core.files.storage import default_storage
        from django.core.urlresolvers import reverse

        data_collection = get_object_or_404(
            DataCollection, id=collection_id)
        collection_fields = DataCollectionField.objects.filter(
            data_collection=data_collection,
        ).order_by('collection_order')
        data_record = get_object_or_404(
            DataRecord,
            data_collection=data_collection,
            id=record_id)

        file_field_names = [
            f.name for f in collection_fields
            if f.field_type == DataCollectionField.FILE]
        for fn in file_field_names:
            _path = data_record.data.get(fn)
            if _path:
                default_storage.delete(_path)

        data_record.delete()
        return redirect(
            reverse(
                'admin:datacollection_records_list',
                args=[data_collection.pk]))

    def export_to_csv_view(self, request, collection_id):
        import csv
        from django.shortcuts import get_object_or_404
        from django.http import HttpResponse

        data_collection = get_object_or_404(
            DataCollection, id=collection_id)
        collection_fields = DataCollectionField.objects.filter(
            data_collection=data_collection,
        ).order_by('collection_order')

        # Create the HttpResponse object with the appropriate CSV header.
        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = \
            'attachment; filename="%s.csv"' % data_collection.title

        writer = csv.writer(response)
        field_names = ['id', 'title', 'active']
        field_names += collection_fields.values_list('name', flat=True)
        writer.writerow(field_names)
        records = data_collection.records.all()

        for record in records:
            data = record.as_dict()
            line = [data.get(i) for i in field_names]
            writer.writerow(line)
        return response

    def import_from_csv_view(self, request, collection_id):
        from django.shortcuts import render, get_object_or_404
        from .forms import DataRecordImportForm, DataRecordForm
        import csv
        data_collection = get_object_or_404(
            DataCollection, id=collection_id)
        collection_fields = DataCollectionField.objects.filter(
            data_collection=data_collection,
        ).order_by('collection_order')

        report = []
        if request.method == 'POST':
            form = DataRecordImportForm(request.POST, request.FILES)
            if form.is_valid():
                # Import data
                _file = request.FILES['source']
                _content = b''
                for chunk in _file.chunks():
                    _content += chunk
                _content = _content.decode('utf-8')
                reader = csv.reader(_content.splitlines())
                i = 0
                headers = []
                for row in reader:
                    i += 1
                    status = ''
                    if i == 1:
                        # First line
                        headers = row
                    else:
                        # Line with data
                        _dict = dict(zip(headers, row))
                        for field in collection_fields:
                            if field.field_type == 'multiple_choice' and \
                                    field.name in _dict.keys():
                                if _dict[field.name] != '':
                                    _dict[field.name] = eval(_dict[field.name])

                        f = DataRecordForm(
                            _dict,
                            data_collection=data_collection,
                            request=request)
                        if f.is_valid():
                            # Data is valid
                            if 'id' in _dict and _dict['id'] != '':
                                id = _dict['id']
                                try:
                                    record = DataRecord.objects.get(
                                        id=id,
                                        data_collection=data_collection)
                                    f.instance = record
                                    record = f.save()
                                    status = 'CHANGED'
                                except DataRecord.DoesNotExist:
                                    record = f.save(commit=False)
                                    record.id = id
                                    record.save()
                                    status = 'CREATED'
                            else:
                                # id not specified
                                record = f.save()
                                status = 'CREATED'
                        else:
                            status = f.errors
                    report.append([status]+row)
        else:
            form = DataRecordImportForm()

        context = dict(
           # Include common variables for rendering the admin template.
           self.admin_site.each_context(request),
           data_collection=data_collection,
           page_title=_('Import from CSV'),
           form=form,
           report=report,
        )
        return render(
            request,
            'admin/data_collections/datacollection/datarecords_import.html',
            context)

    def get_urls(self):
        from django.conf.urls import url
        urls = super().get_urls()
        list_urls = [
            url(
                r'^(?P<collection_id>[\w\d-]+)/datarecords/$',
                self.admin_site.admin_view(self.records_list_view),
                name='datacollection_records_list'),
            url(
                r'^(?P<collection_id>[\w\d-]+)/datarecords/add/$',
                self.admin_site.admin_view(self.record_change_view),
                name='datacollection_records_add'),
            url(
                r'^(?P<collection_id>[\w\d-]+)/datarecords/export-csv/$',
                self.admin_site.admin_view(self.export_to_csv_view),
                name='datacollection_export_to_csv_view'),
            url(
                r'^(?P<collection_id>[\w\d-]+)/datarecords/import-csv/$',
                self.admin_site.admin_view(self.import_from_csv_view),
                name='datacollection_import_from_csv_view'),
            url(
                r'^(?P<collection_id>[\w\d-]+)/datarecords/'
                r'(?P<record_id>[-\w\d]+)/change/$',
                self.admin_site.admin_view(self.record_change_view),
                name='datacollection_records_change'),
            url(
                r'^(?P<collection_id>[\w\d-]+)/datarecords/'
                r'(?P<record_id>[-\w\d]+)/delete/$',
                self.admin_site.admin_view(self.record_delete_view),
                name='datacollection_records_delete'),
        ]
        return list_urls + urls


admin.site.register(DataCollection, DataCollectionAdmin)


# class DataRecordAdmin(admin.ModelAdmin):
#     list_display = ['name', 'data_collection', 'active']
#     list_filter = ['data_collection']
#     search_fields = ['name']
#
# admin.site.register(DataRecord, DataRecordAdmin)


class WebFormAdmin(admin.ModelAdmin):
    list_display = [
        'title', 'data_collection', 'fields', 'hidden_fields', 'show_captcha']
    list_filter = ['data_collection']
    search_fields = ['title']


admin.site.register(WebForm, WebFormAdmin)


class MailHookAdmin(admin.ModelAdmin):
    list_display = [
        'title', 'active', 'data_collection', 'event_type', 'email_template', 'user_group']
    list_filter = ['data_collection']
    search_fields = ['title']
    fieldsets = (
        (None, {
            'fields': [
                'title', 'active', 'data_collection', 'event_type'],
        }),
        (_('Send to'), {
            'fields': ['user_group', 'additional_emails'],
        }),
        (_('Send from'), {
            'fields': ['from_email'],
        }),
        (_('Template'), {
            'fields': [
                'email_template',
            ],
        }),
    )


admin.site.register(MailHook, MailHookAdmin)



import uuid
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.dispatch import receiver
from django.db.models.signals import post_save, pre_save
from adminsortable.models import Sortable
from adminsortable.fields import SortableForeignKey
from django.core.validators import RegexValidator
from django.contrib.postgres.fields import JSONField
from django.contrib.auth.models import Group
from cms.models import CMSPlugin


class DataCollection(models.Model):
    """
    Represents a data collection.
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    title = models.CharField(_('Title'), max_length=255)
    unique_titles = models.BooleanField(
        _('Unique titles'), default=False, blank=True,
        help_text=_('If checked than Title field for the records must be unique.'),
    )

    title_field_label = models.CharField(_('Title field label'), max_length=255, blank=True)
    active_field_label = models.CharField(_('Active field label'), max_length=255, blank=True)
    reversed_ordering = models.BooleanField(_('Reversed ordering'), default=False)

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Changed'), auto_now=True)

    class Meta:
        verbose_name = _('Data collection')
        verbose_name_plural = _('Data collections')
        ordering = ['title']

    def __str__(self) -> str:
        return self.title

    def get_records(self):
        if self.reversed_ordering:
            return self.records.order_by('-added', 'id')
        else:
            return self.records.all()


class DataCollectionFieldManager(models.Manager):
    def admin_list_display(self):
        return self.get_queryset().filter(admin_list_display=True)


class DataCollectionField(Sortable):
    """
    Represents a field for data collection scheme.
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    data_collection = SortableForeignKey(
        DataCollection, verbose_name=_('Data collection'),
        related_name='fields')
    name = models.CharField(
        _('Name'), max_length=255,
        validators=[RegexValidator(
            regex=r'^[a-zA-Z\-_][0-9a-zA-Z\-_]*$',
            message=_("Value may only contain the letters a-z, A-Z, digits, "
                      "minus and underscores, and can't start with a digit."))])
    title = models.CharField(_('Title'), max_length=255)

    # Field types
    STRING = "string"
    EMAIL = "email"
    TEXT = "text"
    INTEGER = "integer"
    BOOLEAN = "boolean"
    FLOAT = "float"
    CHOICE = "choice"
    MULTIPLE_CHOICE = "multiple_choice"
    FILE = "file"
    WYSIWYG = "wysiwyg"
    CODE = "code"
    DATE = "date"
    FIELD_TYPE_CHOICES = (
        (STRING, _("String")),
        (EMAIL, _("Email")),
        (TEXT, _("Multiline text")),
        (INTEGER, _("Integer")),
        (BOOLEAN, _("True / False")),
        (FLOAT, _("Float")),
        (CHOICE, _("Choice")),
        (MULTIPLE_CHOICE, _("Multiple choice")),
        (FILE, _("File")),
        (WYSIWYG, _("WYSIWYG")),
        (CODE, _("Code")),
        (DATE, _("Date")),
    )
    field_type = models.CharField(
        _('Field type'), max_length=20,
        default="string", choices=FIELD_TYPE_CHOICES)
    choices = models.TextField(
        _('Choices'), blank=True, default='',
        help_text=_(
            'Usable for Choice type. Type choices in different lines.'))
    help_text = models.TextField(_('Help text'), blank=True, default='')
    required = models.BooleanField(_('Required'), default=False, blank=True)

    admin_list_display = models.BooleanField(
        _('Admin list display'), default=False, blank=True)

    collection_order = models.PositiveIntegerField(
        default=0, editable=False, db_index=True)

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Changed'), auto_now=True)

    objects = DataCollectionFieldManager()

    class Meta:
        unique_together = ['data_collection', 'name']
        ordering = ['collection_order']
        verbose_name = _('Data collection field')
        verbose_name_plural = _('Data collection fields')

    def __str__(self) -> str:
        return self.title


# Rename fields if required
@receiver(pre_save, sender=DataCollectionField)
def rename_fields_at_records(sender, **kwargs):
    _field = kwargs['instance']
    if _field.id:
        try:
            _old_field = DataCollectionField.objects.get(pk=_field.id)
            if _field.name != _old_field.name:
                # Here we should rename fields for all records
                data_collection = _field.data_collection
                records = data_collection.records.all()
                for r in records.iterator():
                    _data = r.data
                    if _old_field.name in _data:
                        _data[_field.name] = _data[_old_field.name]
                        del _data[_old_field.name]
                        r.data = _data
                        r.save()
        except DataCollectionField.DoesNotExist:
            # Does it is a fixture loading?
            pass


class DataRecord(models.Model):
    """
    Represents a data collection record.
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    data_collection = models.ForeignKey(
        DataCollection,
        verbose_name=_('Data collection'),
        related_name='records')
    title = models.CharField(_('Title'), max_length=255, blank=True)
    active = models.BooleanField(_('Active'), default=True, blank=True)

    data = JSONField(_('Data'), default=dict(), blank=True)

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Last change'), auto_now=True)

    class Meta:
        verbose_name = _('Data record')
        verbose_name_plural = _('Data records')
        ordering = ['added', 'id']

    def __str__(self) -> str:
        return self.title

    def as_dict(self):
        d = {
            'id': self.id,
            'title': self.title,
            'active': self.active,
        }
        d.update(self.data)
        return d


# Work with custom hooks
@receiver(post_save, sender=DataRecord)
def call_mail_hooks(sender, **kwargs):
    _record = kwargs['instance']
    _is_raw = kwargs['raw']
    if _is_raw:
        # Fixture loading
        return None
    data_collection = _record.data_collection
    if kwargs['created']:
        # New record
        mail_hooks = MailHook.objects.filter(
            data_collection=data_collection,
            active=True,
            event_type='new',
        )
        for hook in mail_hooks:
            hook.do_hook_for(_record)
    else:
        # Available record
        mail_hooks = MailHook.objects.filter(
            data_collection=data_collection,
            active=True,
            event_type='changed',
        )
        for hook in mail_hooks:
            hook.do_hook_for(_record)


class WebForm(models.Model):
    """
    Represents a custom web form based on a data collection.
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    title = models.CharField(_('Title'), max_length=255, unique=True)
    data_collection = models.ForeignKey(
        DataCollection,
        verbose_name=_('Data collection'),
        related_name='web_forms')
    fields = models.CharField(
        _('Fields'), max_length=255,
        default='',
        blank=True,
        help_text=_('Comma separated list of field names.'))
    hidden_fields = models.CharField(
        _('Hidden fields'), max_length=255,
        default='',
        blank=True,
        help_text=_(
            'Comma separated list of hidden fields. '
            'Must be subset of the "Fields" list.'))
    show_captcha = models.BooleanField(_('Show captcha'), default=False, blank=True)

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Last change'), auto_now=True)

    class Meta:
        verbose_name = _('Web form')
        verbose_name_plural = _('Web forms')
        ordering = ['title']

    def __str__(self) -> str:
        return self.title


class MailHook(models.Model):
    """
    Represents a mail hook.
    """
    EVENT_CHOICES = (
        ('new', _('New record')),
        ('changed', _('Changed')),
    )

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    title = models.CharField(_('Title'), max_length=255, unique=True)
    active = models.BooleanField(_('Active'), default=True, blank=True)

    data_collection = models.ForeignKey(
        DataCollection,
        verbose_name=_('Data collection'),
        related_name='mail_hooks')
    event_type = models.CharField(
        _('Event type'),
        max_length=25,
        default='new',
        choices=EVENT_CHOICES,
    )
    user_group = models.ForeignKey(
        Group,
        verbose_name=_('User group'),
        null=True, blank=True,
        related_name='mail_hooks',
    )
    additional_emails = models.TextField(
        _('Additional emails'), blank=True, default='',
        help_text=_('Type the emails in different lines.'))

    email_template = models.ForeignKey(
        'pcart_messaging.EmailTemplate', verbose_name=_('Email template'),
        related_name='mail_hooks',
    )
    from_email = models.CharField(
        _('From email'),
        max_length=255,
        default=settings.DEFAULT_FROM_EMAIL,
        help_text=_(
            'You can use {{field_name}} for insert data from data record.')
    )

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Last change'), auto_now=True)

    class Meta:
        verbose_name = _('Mail hook')
        verbose_name_plural = _('Mail hooks')
        ordering = ['title']

    def __str__(self) -> str:
        return self.title

    def _render_text(self, text, context=None):
        from django.template import Context, Template
        _context = Context(context or {})
        template = Template(text)
        return template.render(_context)

    def do_hook_for(self, data_record, extra_context=None):
        from pcart_messaging.utils import send_email_to_group
        _data = data_record.as_dict()

        if self.user_group:
            _group_name = self.user_group.name
        else:
            _group_name = None

        context = {
            'data_record': data_record,
            'data_collection': data_record.data_collection,
            'hook': self,
        }
        context.update(_data)
        if extra_context is not None:
            context.update(extra_context)
        _additional_emails = []
        for e in self.additional_emails.splitlines():
            t = self._render_text(e, context)
            if t:
                _additional_emails.append(t)

        _from_email = self._render_text(
            self.from_email,
            context,
        )

        send_email_to_group(
            template_name=self.email_template.name,
            group_name=_group_name or 'mailhook',
            sender=_from_email,
            context=context,
            extra_recipients=_additional_emails,
        )


# CMS plugins

class WebFormPluginModel(CMSPlugin):
    """
    Represents a plugin with a web form.
    """
    web_form = models.ForeignKey(WebForm, verbose_name=_('Web form'))
    show_title = models.BooleanField(_('Show title'), default=True)
    title = models.CharField(
        _('Title'), max_length=255, default='', blank=True,
        help_text=_('Optional. If not set the web form title will be used.')
    )
    template_name = models.CharField(_('Template name'), default='default', blank=True, max_length=100)

    def __init__(self, *args, **kwargs):
        super(WebFormPluginModel, self).__init__(*args, **kwargs)

    def __str__(self) -> str:
        return str(self.web_form)




from django.db.models import Func


class JSONExtractPathText(Func):
    function = 'json_extract_path_text'
    template = '%(function)s(to_json(%(expressions)s), \'%(path)s\')'


class JSONExtractPathDate(Func):
    function = 'json_extract_path_text'
    template = 'to_date(%(function)s(to_json(%(expressions)s), \'%(path)s\'), \'YYYY-MM-DD\')'



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-06 08:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_data_collections', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='datacollection',
            name='active_field_label',
            field=models.CharField(blank=True, max_length=255, verbose_name='Active field label'),
        ),
        migrations.AddField(
            model_name='datacollection',
            name='title_field_label',
            field=models.CharField(blank=True, max_length=255, verbose_name='Title field label'),
        ),
    ]



from django.contrib.postgres.fields import JSONField
from django.db.models import Lookup


class JSONDateLookup(Lookup):
    """
    Abstract lookup class for working with dates inside JSON field.
    """
    def get_op(self):
        raise NotImplementedError

    def as_sql(self, compiler, connection):
        """Generates a piece of SQL code for date strings comparison."""
        lhs, lhs_params = self.process_lhs(compiler, connection)
        rhs, rhs_params = self.process_rhs(compiler, connection)
        params = lhs_params + rhs_params
        return 'to_date(%s #>> \'{}\', \'YYYY-MM-DD\') %s to_date(trim(both \'"\' from %s), \'YYYY-MM-DD\')' % (
            lhs, self.get_op(), rhs), params


class JSONDateLTLookup(JSONDateLookup):
    """
    'Less then' lookup for dates inside JSON fields.
    """
    lookup_name = 'json_date_lt'

    def get_op(self):
        return '<'


class JSONDateGTLookup(JSONDateLookup):
    """
    'Greater then' lookup for dates inside JSON fields.
    """
    lookup_name = 'json_date_gt'

    def get_op(self):
        return '>'


JSONField.register_lookup(JSONDateLTLookup)
JSONField.register_lookup(JSONDateGTLookup)



import re
from django import template
from django.utils.translation import ugettext as _
from django.utils.safestring import mark_safe
from django.core.cache import cache
from ..models import (
    DataCollection,
    DataCollectionField,
    DataRecord,
)

register = template.Library()


@register.inclusion_tag('datacollections/_field_value.html')
def render_field_value_as_html(
        data_record, field, image_width=100, domain=''):
    if isinstance(field, DataCollectionField):
        field_obj = field
    else:
        field_obj = data_record.data_collection.fields.get(name=field)
    _data = data_record.as_dict()
    return {
        'data_record': data_record,
        'field_obj': field_obj,
        'value': _data.get(field_obj.name),
        'image_width': image_width,
        'domain': domain,
    }


@register.simple_tag()
def list_of_dicts(objects_items):
    return [i.as_dict() for i in objects_items]


@register.filter
def data_values(queryset):
    return queryset.values()


@register.filter
def values_of(sequence, argument):
    return [i.get(argument) for i in sequence]


@register.filter
def get_item(sequence, index):
    return sequence.get(index)


@register.filter
def file_name(fullpath):
    import os
    return os.path.split(fullpath)[-1]


@register.filter
def is_image(fullpath):
    import os
    ext = os.path.splitext(fullpath)[-1].lower()
    return ext in ['.png', '.gif', '.jpg', '.jpeg', '.svg']



import re
from django import template
from django.utils.translation import ugettext_lazy as _


register = template.Library()


# Copied from https://www.djangosnippets.org/snippets/9/

class ExprNode(template.Node):
    def __init__(self, expr_string, var_name):
        self.expr_string = expr_string
        self.var_name = var_name

    def render(self, context):
        from django.template.context import RequestContext, Context
        try:
            clist = list(context)
            clist.reverse()
            d = {}
            d['_'] = _
            for c in clist:
                if isinstance(c, RequestContext) or isinstance(c, Context):
                    for item in c:
                        if isinstance(item, dict):
                            d.update(item)
                else:
                    d.update(c)
            if self.var_name:
                context[self.var_name] = eval(self.expr_string, d)
                return ''
            else:
                return str(eval(self.expr_string, d))
        except:
            raise


r_expr = re.compile(r'(.*?)\s+as\s+(\w+)', re.DOTALL)


def do_expr(parser, token):
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires arguments" % token.contents[0])
    m = r_expr.search(arg)
    if m:
        expr_string, var_name = m.groups()
    else:
        if not arg:
            raise template.TemplateSyntaxError(
                "%r tag at least require one argument" % tag_name)

        expr_string, var_name = arg, None
    return ExprNode(expr_string, var_name)


do_expr = register.tag('expr', do_expr)


class ImportObjectNode(template.Node):
    def __init__(self, import_string, var_name):
        self.import_string = import_string
        self.var_name = var_name

    def render(self, context):
        from apps.core.utils import load_object
        try:
            if self.var_name:
                context[self.var_name] = load_object(self.import_string)
                return ''
            else:
                return str(load_object(self.import_string))
        except:
            raise


r_expr = re.compile(r'(.*?)\s+as\s+(\w+)', re.DOTALL)


def do_load_object(parser, token):
    """
    Syntax:
    {% load_object apps.equipment.models.Equipment as equipment %}
    """
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires arguments" % token.contents[0])
    m = r_expr.search(arg)
    if m:
        expr_string, var_name = m.groups()
    else:
        if not arg:
            raise template.TemplateSyntaxError(
                "%r tag at least require one argument" % tag_name)

        expr_string, var_name = arg, None
    return ImportObjectNode(expr_string, var_name)


do_load_object = register.tag('load_object', do_load_object)



from django.utils.translation import ugettext as _
from django.utils.safestring import mark_safe
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from .models import WebFormPluginModel
from .forms import WebFormPluginForm


class WebFormPluginPublisher(CMSPluginBase):
    model = WebFormPluginModel  # model where plugin data are saved
    form = WebFormPluginForm
    module = _("Data collections")
    name = _("Web form")  # name of the plugin in the interface
    render_template = "datacollections/plugins/web_form_plugin.html"

    def render(self, context, instance, placeholder):
        _context = {}
        _context.update({'instance': instance})
        return _context

plugin_pool.register_plugin(WebFormPluginPublisher)



import json
from django.shortcuts import (
    redirect,
    render,
    get_object_or_404,
)
from django.views.decorators.http import require_POST
from django.views.decorators.csrf import csrf_exempt
from django.contrib.sites.shortcuts import get_current_site
from django.http import HttpResponseForbidden, HttpResponse
from django.conf import settings
from .models import DataCollection, DataRecord, WebForm
from .forms import DataRecordForm, DataRecordSiteForm


def show_web_form(request):
    """
    Returns a form code via AJAX with specified template.
    """
    if not request.is_ajax():
        return HttpResponseForbidden('AJAX request required.')

    # Check the form title
    form_title = request.GET.get('title')
    if form_title is None:
        return HttpResponseForbidden('You should set the "title" argument.')

    # Check the form template
    view = request.GET.get('view', 'default')
    template_name = settings.PCART_WEB_FORM_TEMPLATES.get(view)
    if template_name is None:
        return HttpResponseForbidden('You should specify the correct "view" argument value.')

    web_form = get_object_or_404(WebForm, title__exact=form_title)
    form = DataRecordSiteForm(
        request=request,
        web_form=web_form,
    )

    target_id = request.GET.get('target-id')
    context = {
        'web_form': web_form,
        'form': form,
        'view': view,
        'form_result': False,
        'target_id': '#%s' % target_id or '',
    }
    return render(request, template_name, context)


@require_POST
@csrf_exempt
def data_collection_default_action(request):
    from urllib.parse import urlencode
    http_host = request.META.get('HTTP_HOST')
    current_site = get_current_site(request).domain
    if http_host != current_site:
        return HttpResponseForbidden('Request does not allowed.')

    web_form = None
    data_collection = None
    if 'web-form' in request.POST:
        web_form = get_object_or_404(
            WebForm,
            title__exact=request.POST['web-form']
        )
        data_collection = web_form.data_collection
    else:
        return HttpResponseForbidden('"web-form" attribute is required')

    if 'record-id' in request.POST:
        instance = get_object_or_404(
            DataRecord,
            data_collection=data_collection,
            id=request.POST['record-id'],
        )
    else:
        instance = None

    if instance:
        form = DataRecordSiteForm(
            request.POST,
            request.FILES,
            instance=instance,
            web_form=web_form,
            request=request,
        )
    else:
        form = DataRecordSiteForm(
            request.POST,
            request.FILES,
            web_form=web_form,
            request=request,
        )

    # Check the form template
    view = request.POST.get('view', 'default')
    template_name = settings.PCART_WEB_FORM_TEMPLATES.get(view)
    if template_name is None:
        return HttpResponseForbidden('You should specify the correct "view" argument value.')

    valid = False
    form_result = False
    if form.is_valid():
        valid = True
        instance = form.save()
        form_result = True
    else:
        instance = None
        # _errors = form.errors
        # errors_dict = dict()
        # for e in _errors.keys():
        #     errors_dict.update({e: _errors[e]})
        # print(errors_dict)
        # _get_args = urlencode(errors_dict)
        # print(_get_args)

    target_id = request.POST.get('target-id')
    context = {
        'web_form': web_form,
        'form': form,
        'view': view,
        'instance': instance,
        'valid': valid,
        'form_result': form_result,
        'target_id': '#%s' % target_id or '',
    }
    return render(request, template_name, context)




from django.conf.urls import url
from .views import (
    show_web_form,
    data_collection_default_action,
)

urlpatterns = [
    url(
        r'^webform-show/$',
        show_web_form,
        name='show_web_form'),
    url(
        r'^webform-default-action/$',
        data_collection_default_action,
        name='data_collection_default_action'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-02 19:36
from __future__ import unicode_literals

import adminsortable.fields
import django.contrib.postgres.fields.jsonb
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('pcart_messaging', '0001_initial'),
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='DataCollection',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('unique_titles', models.BooleanField(default=False, help_text='If checked than Title field for the records must be unique.', verbose_name='Unique titles')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Changed')),
            ],
            options={
                'verbose_name_plural': 'Data collections',
                'verbose_name': 'Data collection',
                'ordering': ['title'],
            },
        ),
        migrations.CreateModel(
            name='DataCollectionField',
            fields=[
                ('order', models.PositiveIntegerField(db_index=True, default=0, editable=False)),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255, validators=[django.core.validators.RegexValidator(message="Value may only contain the letters a-z, A-Z, digits, minus and underscores, and can't start with a digit.", regex='^[a-zA-Z\\-_][0-9a-zA-Z\\-_]*$')], verbose_name='Name')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('field_type', models.CharField(choices=[('string', 'String'), ('email', 'Email'), ('text', 'Multiline text'), ('integer', 'Integer'), ('boolean', 'True / False'), ('float', 'Float'), ('choice', 'Choice'), ('multiple_choice', 'Multiple choice'), ('file', 'File'), ('wysiwyg', 'WYSIWYG'), ('code', 'Code'), ('date', 'Date')], default='string', max_length=20, verbose_name='Field type')),
                ('choices', models.TextField(blank=True, default='', help_text='Usable for Choice type. Type choices in different lines.', verbose_name='Choices')),
                ('help_text', models.TextField(blank=True, default='', verbose_name='Help text')),
                ('required', models.BooleanField(default=False, verbose_name='Required')),
                ('admin_list_display', models.BooleanField(default=False, verbose_name='Admin list display')),
                ('collection_order', models.PositiveIntegerField(db_index=True, default=0, editable=False)),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Changed')),
                ('data_collection', adminsortable.fields.SortableForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='fields', to='pcart_data_collections.DataCollection', verbose_name='Data collection')),
            ],
            options={
                'verbose_name_plural': 'Data collection fields',
                'ordering': ['collection_order'],
                'verbose_name': 'Data collection field',
            },
        ),
        migrations.CreateModel(
            name='DataRecord',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(blank=True, max_length=255, verbose_name='Title')),
                ('active', models.BooleanField(default=True, verbose_name='Active')),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default={}, verbose_name='Data')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Last change')),
                ('data_collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='records', to='pcart_data_collections.DataCollection', verbose_name='Data collection')),
            ],
            options={
                'verbose_name_plural': 'Data records',
                'verbose_name': 'Data record',
                'ordering': ['added', 'id'],
            },
        ),
        migrations.CreateModel(
            name='MailHook',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(max_length=255, unique=True, verbose_name='Title')),
                ('active', models.BooleanField(default=True, verbose_name='Active')),
                ('event_type', models.CharField(choices=[('new', 'New record'), ('changed', 'Changed')], default='new', max_length=25, verbose_name='Event type')),
                ('additional_emails', models.TextField(blank=True, default='', help_text='Type the emails in different lines.', verbose_name='Additional emails')),
                ('from_email', models.CharField(default='no-reply@p-cart.com', help_text='You can use {{field_name}} for insert data from data record.', max_length=255, verbose_name='From email')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Last change')),
                ('data_collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='mail_hooks', to='pcart_data_collections.DataCollection', verbose_name='Data collection')),
                ('email_template', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='mail_hooks', to='pcart_messaging.EmailTemplate', verbose_name='Email template')),
                ('user_group', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='mail_hooks', to='auth.Group', verbose_name='User group')),
            ],
            options={
                'verbose_name_plural': 'Mail hooks',
                'verbose_name': 'Mail hook',
                'ordering': ['title'],
            },
        ),
        migrations.CreateModel(
            name='WebForm',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(max_length=255, unique=True, verbose_name='Title')),
                ('fields', models.CharField(blank=True, default='', help_text='Comma separated list of field names.', max_length=255, verbose_name='Fields')),
                ('hidden_fields', models.CharField(blank=True, default='', help_text='Comma separated list of hidden fields. Must be subset of the "Fields" list.', max_length=255, verbose_name='Hidden fields')),
                ('show_captcha', models.BooleanField(default=False, verbose_name='Show captcha')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Last change')),
                ('data_collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='web_forms', to='pcart_data_collections.DataCollection', verbose_name='Data collection')),
            ],
            options={
                'verbose_name_plural': 'Web forms',
                'verbose_name': 'Web form',
                'ordering': ['title'],
            },
        ),
        migrations.AlterUniqueTogether(
            name='datacollectionfield',
            unique_together=set([('data_collection', 'name')]),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class PCartDataCollectionsConfig(AppConfig):
    name = 'pcart_data_collections'
    verbose_name = _('Data collections')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-06 12:36
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_data_collections', '0002_auto_20171006_1125'),
    ]

    operations = [
        migrations.AddField(
            model_name='datacollection',
            name='reversed_ordering',
            field=models.BooleanField(default=False, verbose_name='Reversed ordering'),
        ),
    ]
